/**************************************************************************//**
*   @file   AD5684R.c
*   @brief  Implementation of AD5684R Driver for Microblaze processor.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define AD5684R_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "ad5684r.h"
#include <altera_avalon_pio_regs.h>	// PIO access macros
#include <altera_avalon_spi.h>		// SPI
#include <unistd.h>					// usleep();
#include <system.h>

#if(USE_AD5684R == 1)
/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char   txBuffer[4]     = {0,}; // Tx SPI Buffer.
unsigned char   rxBuffer[4]     = {0,}; // Rx SPI Buffer.

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
volatile char*	ad5684r_dacAEnable	= &AD5684R_Data[0].chData;
volatile char*  ad5684r_dacBEnable  = &AD5684R_Data[1].chData;
volatile char*  ad5684r_dacCEnable	= &AD5684R_Data[2].chData;
volatile char*	ad5684r_dacDEnable  = &AD5684R_Data[3].chData;
volatile int*	ad5684r_dacASlider	= &AD5684R_Data[4].iData;
volatile int*	ad5684r_dacBSlider	= &AD5684R_Data[5].iData;
volatile int*	ad5684r_dacCSlider  = &AD5684R_Data[6].iData;
volatile int*	ad5684r_dacDSlider	= &AD5684R_Data[7].iData;
volatile char*	ad5684r_softReset	= &AD5684R_Data[8].chData;
volatile char*	ad5684r_powerDown	= &AD5684R_Data[9].chData;

int prevDacA = 0;
int prevDacB = 0;
int prevDacC = 0;
int prevDacD = 0;

char prevDacAEn = 0;
char prevDacBEn = 0;
char prevDacCEn = 0;
char prevDacDEn = 0;

char prevPowerDown = 0;

/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for AD5684R Board.
 *
 * @param None.
 *
 * @return TRUE or FALSE.
******************************************************************************/
int AD5684R_Init( void )

{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	// setup SPI interface
	// setup SPI interface
	SetCtrl(AD5684R_SPI_CFG);

	// GPIO0 = 1 => No LDAC
	// GPIO1 = 1 => No RESET
	// GPIO2 = 1 => AD5684R
	// GPIO3 = 0
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, 0x0F);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, 0x07);

    cfgValue = CMD_HW_LDAC_MSK | 0x0F;
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;

	// Transfer data via SPI
	alt_avalon_spi_command(AD5684R_SPI, 2, 3, txBuffer, 0, NULL, 0);

	return(SUCCESS);
}

/**************************************************************************//**
 * @brief Writes data to the input register.
 *
 * @param dacSel - which DAC is selected.
 *
 * @param txData - Data to be written.
 *
 * @return None.
******************************************************************************/
void AD5684R_WriteInputReg(int dacSel, int txData)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select Command Type and DAC, add desired Data
	cfgValue = CMD_WR_INPUT | dacSel | ((txData & 0xFFFF) << 4);
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5684R_SPI, 0, 3, txBuffer, 0, NULL, 0);
}

/**************************************************************************//**
 * @brief Update data to the registers.
 *
 * @param dacSel - which DAC is selected.
 *
 * @return None.
******************************************************************************/
void AD5684R_UpdateDacReg(int dacSel)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command type and DAC
	cfgValue = CMD_UP_DAC | dacSel;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = 0x00;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5684R_SPI, 0, 3, txBuffer, 0, NULL, 0);
}

/**************************************************************************//**
 * @brief Writes data to the input register and update its contents.
 *
 * @param dacSel - which DAC is selected.
 *
 * @param txData - Data to be written.
 *
 * @return None.
******************************************************************************/

void AD5684R_WriteUpdateDacReg(int dacSel, int txData)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select Command Type and DAC, add desired Data
	cfgValue = CMD_WR_UP_DAC | dacSel | ((txData & 0xFFFF) << 4);
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5684R_SPI, 0, 3, txBuffer, 0, NULL, 0);
}

/**************************************************************************//**
 * @brief Power Up/Down the device
 *
 * @return None.
******************************************************************************/
void AD5684R_PowerUpDown(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_PWR_DAC;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5684R_SPI, 0, 3, txBuffer, 0, NULL, 0);
}

/**************************************************************************//**
 * @brief Perform a software reset.
 *
 * @return None.
******************************************************************************/
void AD5684R_SoftReset(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_SW_RST;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0xFF0000) >> 16;
	txBuffer[1] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[2] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5684R_SPI, 0, 3, txBuffer, 0, NULL, 0);
}


/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return TRUE
******************************************************************************/
int AD5684R_UpdateParameters(void)
{
	if(*ad5684r_softReset != prevPowerDown)
	{
		prevPowerDown = *ad5684r_softReset;
		AD5684R_SoftReset();
	}
	else
	{
		if(*ad5684r_dacASlider != prevDacA)
		{
			prevDacA = *ad5684r_dacASlider;
			AD5684R_WriteUpdateDacReg(DAC_A, *ad5684r_dacASlider);
		}
		if(*ad5684r_dacBSlider != prevDacB)
		{
			prevDacB = *ad5684r_dacBSlider;
			AD5684R_WriteUpdateDacReg(DAC_B, *ad5684r_dacBSlider);
		}
		if(*ad5684r_dacCSlider != prevDacC)
		{
			prevDacC = *ad5684r_dacCSlider;
			AD5684R_WriteUpdateDacReg(DAC_C, *ad5684r_dacCSlider);
		}
		if(*ad5684r_dacDSlider != prevDacD)
		{
			prevDacD = *ad5684r_dacDSlider;
			AD5684R_WriteUpdateDacReg(DAC_D, *ad5684r_dacDSlider);
		}
	}
    return(SUCCESS);
}
#endif
